@@ -4,7 +4,7 @@
  * Copyright (C) 2007-2010 Kaj-Michael Lang
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
+ * it under the terms of the GNU General Publi"c License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
@@ -92,6 +92,9 @@ struct _DictoWidgetPriv
 	GtkWidget *level;
 	gfloat ndb;
 
+	gdouble pos;
+	gdouble len;
+
 	gboolean allow_delete;
 	gboolean seeking;
 };
@@ -128,10 +131,11 @@ static guint signals[LAST_SIGNAL]={ 0 };
 static void dicto_widget_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void dicto_widget_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec   *pspec);
 
-static gboolean dicto_widget_record_cb(GtkWidget *widget, gpointer data);
+static gboolean dicto_widget_update_view_mode_cb(Dicto *d, gpointer data);
+static gboolean dicto_widget_delete_cb(GtkWidget *widget, gpointer data);
 static gboolean dicto_widget_play_cb(GtkWidget *widget, gpointer data);
 static gboolean dicto_widget_stop_cb(GtkWidget *widget, gpointer data);
-static gboolean dicto_widget_delete_cb(GtkWidget *widget, gpointer data);
+static gboolean dicto_widget_record_cb(GtkWidget *widget, gpointer data);
 static gboolean dicto_widget_ready_cb(GtkWidget *widget, gpointer data);
 static gboolean dicto_widget_level_cb(GtkWidget *widget, gdouble r, gdouble d, gdouble p, gpointer data);
 static gboolean dicto_widget_error_cb(GtkWidget *widget, gpointer error, gpointer data);
@@ -142,13 +146,13 @@ static void dicto_widget_tree_tap_note_cb(GtkTreeView *tree_view, GtkTreePath *p
 #endif
 static void dicto_widget_tree_select_note_cb(GtkTreeSelection *ts, gpointer data);
 
-static gboolean dicto_widget_position_cb(Dicto *d, gdouble left, gpointer data);
+static gboolean dicto_widget_position_cb(Dicto *d, gdouble pos, gdouble len, gdouble left, gpointer data);
 
 static void dicto_widget_position_seek_cb(GtkRange *range, gpointer data);
 static gboolean dicto_widget_position_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data);
 static gboolean dicto_widget_position_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data);
 
-static void dicto_widget_set_length(DictoWidget *dw, gdouble cur_secs, gdouble tot_secs);
+static void dicto_widget_set_length(DictoWidget *dw);
 static void dicto_widget_update_view_mode(DictoWidgetPriv *priv);
 
 /******************************************************************************/
@@ -250,7 +254,7 @@ priv->lbl_len=gtk_label_new("");
 priv->btn_record=gtk_button_new();
 gtk_button_set_image(GTK_BUTTON(priv->btn_record), gtk_image_new_from_stock(GTK_STOCK_MEDIA_RECORD, GTK_ICON_SIZE_BUTTON));
 
-priv->btn_play=gtk_button_new();
+priv->btn_play=gtk_toggle_button_new();
 gtk_button_set_image(GTK_BUTTON(priv->btn_play), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON));
 
 priv->btn_stop=gtk_button_new();
@@ -319,13 +323,8 @@ gtk_box_pack_start(GTK_BOX(priv->hbox), v, FALSE, FALSE, HILDON_MARGIN_DEFAULT);
 gtk_widget_show(priv->vbox);
 gtk_box_pack_start(GTK_BOX(dw), priv->vbox, TRUE, TRUE, 0);
 
-gtk_widget_set_sensitive(priv->btn_play, FALSE);
-gtk_widget_set_sensitive(priv->btn_record, FALSE);
-gtk_widget_set_sensitive(priv->btn_stop, FALSE);
-gtk_widget_set_sensitive(priv->btn_delete, FALSE);
-
+g_signal_connect(G_OBJECT(priv->btn_play), "toggled", G_CALLBACK(dicto_widget_play_cb), dw);
 g_signal_connect(G_OBJECT(priv->btn_record), "clicked", G_CALLBACK(dicto_widget_record_cb), dw);
-g_signal_connect(G_OBJECT(priv->btn_play), "clicked", G_CALLBACK(dicto_widget_play_cb), dw);
 g_signal_connect(G_OBJECT(priv->btn_stop), "clicked", G_CALLBACK(dicto_widget_stop_cb), dw);
 g_signal_connect(G_OBJECT(priv->btn_delete), "clicked", G_CALLBACK(dicto_widget_delete_cb), dw);
 
@@ -339,6 +338,8 @@ g_signal_connect(G_OBJECT(priv->position), "button-press-event", G_CALLBACK(dict
 g_signal_connect(G_OBJECT(priv->position), "button-release-event", G_CALLBACK(dicto_widget_position_release_cb), dw);
 
 g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(dicto_widget_tree_select_note_cb), dw);
+
+dicto_widget_update_view_mode(priv);
 }
 
 static void
@@ -353,11 +354,10 @@ priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_if_fail(priv);
 g_return_if_fail(priv->dicto);
 
+priv->pos=gtk_range_get_value(range);
 s=dicto_get_state(priv->dicto);
-if ((s==DICTO_STATE_PLAYING || s==DICTO_STATE_READY) && priv->seeking) {
-	if (dicto_seek(priv->dicto, gtk_range_get_value(range)))
-		dicto_widget_position_cb(priv->dicto, 0, dw);
-}
+if ((s==DICTO_STATE_PLAYING || s==DICTO_STATE_PAUSED || s==DICTO_STATE_READY) && priv->seeking)
+	dicto_seek(priv->dicto, priv->pos);
 }
 
 static gboolean
@@ -483,37 +483,56 @@ gtk_tree_model_get(model, &iter, NOTE_ID, &file, -1);
 
 g_debug("File %s activated", file);
 
+#if 1
+gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->btn_play), TRUE);
+#else
 if (!dicto_play(priv->dicto, file))
 	dicto_widget_update_view_mode(priv);
+#endif
 
 g_free(file);
 }
 
 static gboolean
-dicto_widget_position_cb(Dicto *d, gdouble left, gpointer data)
+dicto_widget_seek_cb(Dicto *d, gdouble pos, gpointer data)  
 {
-gdouble len=0, pos=0;
 DictoWidget *dw=(DictoWidget *)data;
-DictoWidgetPriv *priv;
+DictoWidgetPriv *p;
 
 g_return_val_if_fail(IS_DICTO(d), TRUE);
 g_return_val_if_fail(IS_DICTO_WIDGET(dw), TRUE);
-priv=DICTO_WIDGET_GET_PRIVATE(dw);
+p=DICTO_WIDGET_GET_PRIVATE(dw);
 
-if (dicto_get_position(d, &len, &pos)) {
-	g_debug("%f/%f", pos, len);
-	dicto_widget_set_length(dw, pos, len);
-	if (len>0 || pos>0)
-		gtk_range_set_range(GTK_RANGE(priv->position), 0.0, len>=pos ? len : pos);
-	if (dicto_get_state(priv->dicto)==DICTO_STATE_PLAYING && !priv->seeking)
-		gtk_range_set_value(GTK_RANGE(priv->position), pos);
+p->pos=pos;
+
+dicto_widget_set_length(dw);
+return FALSE;
 }
 
+static gboolean
+dicto_widget_position_cb(Dicto *d, gdouble pos, gdouble len, gdouble left, gpointer data)
+{
+DictoWidget *dw=(DictoWidget *)data;
+DictoWidgetPriv *p;
+DictoState s;
+
+g_return_val_if_fail(IS_DICTO(d), TRUE);
+g_return_val_if_fail(IS_DICTO_WIDGET(dw), TRUE);
+p=DICTO_WIDGET_GET_PRIVATE(dw);
+
+p->pos=pos;
+p->len=len;
+if (len>0.0 || pos>0.0)
+	gtk_range_set_range(GTK_RANGE(p->position), 0.0, len>=pos ? len : pos);
+if (!p->seeking && len>=pos)
+	gtk_range_set_value(GTK_RANGE(p->position), pos);
+dicto_widget_set_length(dw);
+
 return FALSE;
 }
 
 static gboolean
-dicto_widget_playing_cb(Dicto *d, gpointer data)
+dicto_widget_update_view_mode_cb(Dicto *d, gpointer data)
 {
 DictoWidget *dw=(DictoWidget *)data;
 DictoWidgetPriv *priv;
@@ -526,50 +545,6 @@ dicto_widget_update_view_mode(priv);
 return TRUE;
 }
 
-static gboolean
-dicto_widget_recording_cb(Dicto *d, gpointer data)
-{
-DictoWidget *dw=(DictoWidget *)data;
-DictoWidgetPriv *priv;
-
-g_return_val_if_fail(d, TRUE);
-g_return_val_if_fail(data, TRUE);
-
-priv=DICTO_WIDGET_GET_PRIVATE(dw);
-dicto_widget_update_view_mode(priv);
-return TRUE;
-}
-
-static gboolean
-dicto_widget_stopped_record_cb(Dicto *d, gpointer data)
-{
-DictoWidget *dw=(DictoWidget *)data;
-DictoWidgetPriv *priv;
-
-g_return_val_if_fail(d, TRUE);
-g_return_val_if_fail(data, TRUE);
-
-priv=DICTO_WIDGET_GET_PRIVATE(dw);
-
-dicto_widget_update_view_mode(priv);
-return TRUE;
-}
-
-static gboolean
-dicto_widget_stopped_play_cb(Dicto *d, gpointer data)
-{
-DictoWidget *dw=(DictoWidget *)data;
-DictoWidgetPriv *priv;
-
-g_return_val_if_fail(d, TRUE);
-g_return_val_if_fail(data, TRUE);
-
-priv=DICTO_WIDGET_GET_PRIVATE(dw);
-
-dicto_widget_update_view_mode(priv);
-return TRUE;
-}
-
 static void
 dicto_widget_append_file(gpointer key, gpointer value, gpointer data)
 {
@@ -623,10 +598,12 @@ switch (prop_id) {
 			/* XXX: disconnect signals */
 			dicto_stop(priv->dicto);
 			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_position_cb), dw);
-			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_playing_cb), dw);
-			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_recording_cb), dw);
-			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_stopped_play_cb), dw);
-			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_stopped_record_cb), dw);
+			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_seek_cb), dw);
+			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
 			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_updatelist_cb), dw);
 			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_ready_cb), dw);
 			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_level_cb), dw);
@@ -638,10 +615,12 @@ switch (prop_id) {
 		dicto_widget_updatelist_cb(priv->dicto, dw);
 		dicto_widget_update_view_mode(priv);
 		g_signal_connect(G_OBJECT(priv->dicto), "position", G_CALLBACK(dicto_widget_position_cb), dw);
-		g_signal_connect(G_OBJECT(priv->dicto), "playing", G_CALLBACK(dicto_widget_playing_cb), dw);
-		g_signal_connect(G_OBJECT(priv->dicto), "recording", G_CALLBACK(dicto_widget_recording_cb), dw);
-		g_signal_connect(G_OBJECT(priv->dicto), "stopped-play", G_CALLBACK(dicto_widget_stopped_play_cb), dw);
-		g_signal_connect(G_OBJECT(priv->dicto), "stopped-record", G_CALLBACK(dicto_widget_stopped_record_cb), dw);
+		g_signal_connect(G_OBJECT(priv->dicto), "seeked", G_CALLBACK(dicto_widget_seek_cb), dw);
+		g_signal_connect(G_OBJECT(priv->dicto), "playing", G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+		g_signal_connect(G_OBJECT(priv->dicto), "paused-play", G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+		g_signal_connect(G_OBJECT(priv->dicto), "recording", G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+		g_signal_connect(G_OBJECT(priv->dicto), "stopped-play", G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
+		g_signal_connect(G_OBJECT(priv->dicto), "stopped-record", G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "refresh", G_CALLBACK(dicto_widget_updatelist_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "ready", G_CALLBACK(dicto_widget_ready_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "level", G_CALLBACK(dicto_widget_level_cb), dw);
@@ -712,13 +691,16 @@ gtk_label_set_markup(GTK_LABEL(priv->lbl_len), buffer);
 static void
 dicto_widget_update_view_mode(DictoWidgetPriv *priv)
 {
-DictoState mode;
+DictoState mode=DICTO_STATE_STOPPED;
+GtkTreeSelection *selection;
 
 g_return_if_fail(priv);
-g_return_if_fail(priv->dicto);
 
-mode=dicto_get_state(priv->dicto);
-g_debug("VMode: %d", mode);
+if (priv->dicto)
+	mode=dicto_get_state(priv->dicto);
+
+g_debug("Mode: %d", mode);
+
 switch (mode) {
 	case DICTO_STATE_ERROR:
 		/* XXX: What should we do ?*/
@@ -727,17 +709,29 @@ switch (mode) {
 		dicto_widget_position_clear(priv);
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), 0.0);
 		gtk_widget_set_sensitive(priv->btn_play, FALSE);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->btn_play), FALSE);
 		gtk_widget_set_sensitive(priv->btn_record, TRUE);
 		gtk_widget_set_sensitive(priv->btn_stop, FALSE);
 		gtk_widget_set_sensitive(priv->btn_delete, FALSE);
 
 		gtk_widget_set_sensitive(priv->file_tree, TRUE);
 		gtk_widget_set_sensitive(priv->position, FALSE);
+
+		selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->file_tree));
+		gtk_tree_selection_unselect_all(selection);
 		gtk_range_set_value(GTK_RANGE(priv->position), 0.0);
 #ifndef WITH_HILDON_2_2
 		gtk_range_set_update_policy(GTK_RANGE(priv->position), GTK_UPDATE_CONTINUOUS);
 #endif
 	break;
+	case DICTO_STATE_PAUSED:
+		gtk_widget_set_sensitive(priv->btn_record, FALSE);
+		gtk_widget_set_sensitive(priv->btn_play, TRUE);
+		gtk_widget_set_sensitive(priv->btn_stop, TRUE);
+		gtk_widget_set_sensitive(priv->btn_delete, FALSE);
+		gtk_range_set_update_policy(GTK_RANGE(priv->position), GTK_UPDATE_CONTINUOUS);
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), 0.0);
+	break;
 	case DICTO_STATE_READY:
 		dicto_widget_position_set(priv, 0, 0);
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), 0.0);
@@ -759,7 +753,7 @@ switch (mode) {
 #ifndef WITH_HILDON_2_2
 		gtk_range_set_update_policy(GTK_RANGE(priv->position), GTK_UPDATE_DELAYED);
 #endif
-		gtk_widget_set_sensitive(priv->btn_play, FALSE);
+		gtk_widget_set_sensitive(priv->btn_play, mode==DICTO_STATE_PLAYING);
 		gtk_widget_set_sensitive(priv->btn_record, FALSE);
 		gtk_widget_set_sensitive(priv->btn_stop, TRUE);
 		gtk_widget_set_sensitive(priv->btn_delete, FALSE);
@@ -788,36 +782,40 @@ gtk_tree_model_get(model, &iter, NOTE_ID, &file, -1);
 return file;
 }
 
-static void
-dicto_widget_set_length(DictoWidget *dw, gdouble cur_secs, gdouble tot_secs)
+static inline void
+dicto_widget_get_min_sec(gdouble secs, guint *m, gdouble *s)
 {
-DictoWidgetPriv *priv;
-guint min=0;
+*m=0;
+*s=secs;
+if (*s>=60.0) {
+	*m=*s/60.0;
+	*s-=*m*60.0;
+}
+}
+
+static void
+dicto_widget_set_length(DictoWidget *dw)
+{
+DictoWidgetPriv *p;
+guint m;
+gdouble s;
 
 g_return_if_fail(dw);
-priv=DICTO_WIDGET_GET_PRIVATE(dw);
+p=DICTO_WIDGET_GET_PRIVATE(dw);
 
-if (cur_secs<0.0) {
-	dicto_widget_position_clear(priv);
+if (p->pos<0.0) {
+	dicto_widget_position_clear(p);
 	return;
 }
 
-if (cur_secs>=60.0) {
-	min=cur_secs/60.0;
-	cur_secs-=min*60.0;
-}
+dicto_widget_get_min_sec(p->pos, &m, &s);
+dicto_widget_position_set(p, m, s);
 
-dicto_widget_position_set(priv, min, cur_secs);
-
-if (tot_secs<0)
+if (p->len<0.0)
 	return;
-min=0;
-if (tot_secs>=60.0) {
-	min=tot_secs/60.0;
-	tot_secs-=min*60.0;
-}
 
-dicto_widget_length_set(priv, min, tot_secs);
+dicto_widget_get_min_sec(p->len, &m, &s);
+dicto_widget_length_set(p, m, s);
 }
 
 static gboolean
@@ -891,14 +889,18 @@ gchar *f;
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_val_if_fail(priv, TRUE);
 
-f=dicto_widget_get_selected_file(GTK_TREE_VIEW(priv->file_tree));
-if (!f)
-	return TRUE;
+if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))==TRUE) {
+	f=dicto_widget_get_selected_file(GTK_TREE_VIEW(priv->file_tree));
+	if (!f)
+		return TRUE;
 
-if (!dicto_play(priv->dicto, f))
-	dicto_widget_update_view_mode(priv);
+	if (!dicto_play(priv->dicto, f))
+		dicto_widget_update_view_mode(priv);
 
-g_free(f);
+	g_free(f);
+} else {
+	dicto_pause(priv->dicto);
+}
 return TRUE;
 }
 
