@@ -67,6 +67,7 @@ enum {
 	SIGNAL_RECORDING,
 	SIGNAL_STOPPED_PLAY,
 	SIGNAL_STOPPED_RECORD,
+	SIGNAL_PAUSED_PLAY,
 	SIGNAL_ERROR,
 	SIGNAL_NOTES_REFRESH,
 	SIGNAL_READY,
@@ -100,6 +101,8 @@ struct _dicto_pipeline {
 	GstCaps	*caps;
 	gboolean active;
 	gboolean rec;
+	gdouble position;
+	gdouble length;
 	GObject *owner;
 };
 
@@ -386,7 +389,7 @@ g_object_class_install_property(object_class, PROP_STATE, pspec);
 
 signals[SIGNAL_POSITION]=g_signal_new("position", G_OBJECT_CLASS_TYPE(object_class),
 	G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET(DictoClass, position_update),
-	NULL, NULL, g_cclosure_marshal_VOID__DOUBLE, G_TYPE_NONE, 1, G_TYPE_DOUBLE);
+	NULL, NULL, _dicto_VOID__DOUBLE_DOUBLE_DOUBLE, G_TYPE_NONE, 3, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
 
 signals[SIGNAL_PLAYING]=g_signal_new("playing", G_OBJECT_CLASS_TYPE(object_class),
 	G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET(DictoClass, started_play),
@@ -414,6 +417,10 @@ signals[SIGNAL_READY]=g_signal_new("ready", G_OBJECT_CLASS_TYPE(object_class),
 	G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET(DictoClass, ready),
 	NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
+signals[SIGNAL_PAUSED_PLAY]=g_signal_new("paused-play", G_OBJECT_CLASS_TYPE(object_class),
+	G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET(DictoClass, paused_play),
+	NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
 signals[SIGNAL_SEEK]=g_signal_new("seeked", G_OBJECT_CLASS_TYPE(object_class),
 	G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET(DictoClass, ready),
 	NULL, NULL, g_cclosure_marshal_VOID__DOUBLE, G_TYPE_NONE, 1, G_TYPE_DOUBLE);
@@ -435,7 +442,7 @@ dicto_get_state(Dicto *d)
 {
 DictoPrivate *p;
 
-g_return_val_if_fail(IS_DICTO(d), FALSE);
+g_return_val_if_fail(IS_DICTO(d), DICTO_STATE_UNKNOWN);
 p=GET_PRIVATE(d);
 
 return p->state;
@@ -446,26 +453,27 @@ dicto_position_cb(gpointer data)
 {
 Dicto *d=(Dicto *)data;
 DictoPrivate *p;
+gdouble len;
 
 g_return_val_if_fail(IS_DICTO(d), FALSE);
 p=GET_PRIVATE(d);
 
-/* Are we recording for a specific time ? */
-if (p->rforsecs>0) {
-	if (!dicto_get_position(d, NULL, &p->pos)) {
-		g_warning("Failed to get position for timeout recording");
-		return TRUE;
-	}
+if (!dicto_get_position(d, &len, &p->pos)) {
+	g_warning("Failed to get position");
+	return TRUE;
+}
 
+/* Are we recording for a specific time ? */
+if (p->rforsecs>0.0) {
 	if (p->pos>=p->rforsecs)
 		dicto_stop(d);
 	else
-		g_signal_emit(G_OBJECT(data), signals[SIGNAL_POSITION], 0, p->rforsecs-p->pos, NULL);
+		g_signal_emit(G_OBJECT(data), signals[SIGNAL_POSITION], 0, p->pos, len, p->rforsecs-p->pos, NULL);
 
 	return TRUE;
 }
 
-g_signal_emit(G_OBJECT(data), signals[SIGNAL_POSITION], 0, 0, NULL);
+g_signal_emit(G_OBJECT(data), signals[SIGNAL_POSITION], 0, p->pos, len, 0.0, NULL);
 return TRUE;
 }
 
@@ -640,18 +648,15 @@ DictoPrivate *p=GET_PRIVATE(d);
 
 switch (GST_MESSAGE_TYPE(msg)) {
 	case GST_MESSAGE_EOS:
-		np->active=FALSE;
 		if (np->rec) {
+			np->active=FALSE;
 			gst_element_set_state(np->pipeline, GST_STATE_NULL);
 			dicto_position_update(DICTO(np->owner), FALSE);
 			p->state=DICTO_STATE_STOPPED;
 			g_signal_emit(G_OBJECT(d), signals[SIGNAL_STOPPED_RECORD], 0, NULL);
 		} else {
-			gst_element_set_state(np->pipeline, GST_STATE_PAUSED);
-			dicto_position_update(d, FALSE);
-			p->state=DICTO_STATE_STOPPED;
-			g_signal_emit(np->owner, signals[SIGNAL_STOPPED_PLAY], 0, NULL);
-			dicto_seek(d, 0.0);
+			if (dicto_pause(d))
+				dicto_seek(d, 0.0);
 		}
 	break;
 	case GST_MESSAGE_ERROR:
@@ -670,7 +675,7 @@ switch (GST_MESSAGE_TYPE(msg)) {
 			return TRUE;
 		gst_message_parse_state_changed(msg, &oldstate, &newstate, &pending);
 		if (newstate==GST_STATE_PAUSED && oldstate==GST_STATE_READY)
-			g_signal_emit(DICTO(np->owner), signals[SIGNAL_POSITION], 0, NULL);
+			dicto_position_cb(np->owner);
 		if (np->rec && newstate==GST_STATE_READY && oldstate==GST_STATE_PLAYING) {
 			gst_element_set_state(np->pipeline, GST_STATE_NULL);
 			g_object_set(G_OBJECT(np->sink), "location", "", NULL);
@@ -913,6 +918,9 @@ np->src=src;
 np->sink=sink;
 np->level=level;
 
+np->position=0.0;
+np->length=0.0;
+
 /* set current format */
 p->ext=f->ext;
 p->format=nf;
@@ -1179,6 +1187,12 @@ dicto_refresh_list(DICTO(data));
 return FALSE;
 }
 
+static gboolean
+dicto_seek_pipeline(dicto_pipeline *p)
+{
+return gst_element_seek(p->pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, (gint64)(p->position*GST_SECOND), GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
+}
+
 /**
  * dicto_play:
  * @d: A #Dicto object
@@ -1199,16 +1213,17 @@ g_return_val_if_fail(file, FALSE);
 if (!p->play.pipeline)
 	return FALSE;
 
-if (p->play.active==TRUE)
-	return TRUE;
-
 if (p->record.active==TRUE)
 	return FALSE;
 
-p->rforsecs=-1;
+p->rforsecs=-1.0;
 
 dicto_set_filename(d, p->play.src, file);
 
+if (p->play.position>0.0 && dicto_seek_pipeline(&p->play)) {
+	g_warning("Seek failed");
+}
+
 if (gst_element_set_state(p->play.pipeline, GST_STATE_PLAYING)==GST_STATE_CHANGE_FAILURE) {
 	g_warning("Failed to play file %s\n", p->cfile);
 	dicto_set_filename(d, p->play.src, NULL);
@@ -1257,6 +1272,9 @@ if (p->tags)
 	g_hash_table_destroy(p->tags);
 p->tags=g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
+p->play.position=0.0;
+p->play.length=0.0;
+
 gst_element_set_state(p->play.pipeline, GST_STATE_NULL);
 dicto_set_filename(d, p->play.src, file);
 if (gst_element_set_state(p->play.pipeline, GST_STATE_PAUSED)==GST_STATE_CHANGE_FAILURE) {
@@ -1273,6 +1291,7 @@ if (gst_element_get_state(p->play.pipeline, &current, NULL, GST_SECOND/2)==GST_S
 
 p->state=DICTO_STATE_READY;
 g_signal_emit(d, signals[SIGNAL_READY], 0, NULL);
+dicto_position_cb(d);
 return TRUE;
 }
 
@@ -1286,11 +1305,11 @@ return TRUE;
  *
  * Returns: %TRUE if seeking was successfull.
  */
-gboolean dicto_seek(Dicto *d, gdouble pos)
+gboolean
+dicto_seek(Dicto *d, gdouble pos)
 {
 DictoPrivate *p=GET_PRIVATE(d);
-GstState current;
-GstState pending;
+GstState current, pending;
 
 g_return_val_if_fail(d, FALSE);
 g_return_val_if_fail(pos>=0.0, FALSE);
@@ -1298,18 +1317,32 @@ g_return_val_if_fail(pos>=0.0, FALSE);
 if (!p->play.pipeline)
 	return FALSE;
 
-gst_element_get_state(p->play.pipeline, &current, &pending, GST_CLOCK_TIME_NONE);
-if (current==GST_STATE_NULL) {
-	g_debug("Not in seekable state: %d", current);
+if (gst_element_get_state(p->play.pipeline, &current, &pending, GST_CLOCK_TIME_NONE)==GST_STATE_CHANGE_FAILURE) {
+	g_warning("Failed to get current state for seeking");
 	return FALSE;
 }
 
-if (!gst_element_seek(p->play.pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, (gint64)(pos*GST_SECOND), GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE)) {
+/* Seek directly only if we active, dicto_play() will handle seeking from !PLAYING */
+switch (current) {
+	case GST_STATE_VOID_PENDING:
+	case GST_STATE_NULL:
+		p->play.position=0.0;
+		return FALSE;
+	case GST_STATE_PAUSED:
+		p->play.position=pos;
+		g_signal_emit(d, signals[SIGNAL_SEEK], 0, pos, NULL);
+		return TRUE;
+	case GST_STATE_READY:
+	case GST_STATE_PLAYING:
+		p->play.position=pos;
+	break;
+}
+
+if (!dicto_seek_pipeline(&p->play)) {
 	g_warning("Seek failed");
 	return FALSE;
 }
 
-g_debug("Seek to %f", pos);
 g_signal_emit(d, signals[SIGNAL_SEEK], 0, pos, NULL);
 
 return TRUE;
@@ -1350,6 +1383,9 @@ if (p->tags) {
 	p->tags=NULL;
 }
 
+p->record.position=0.0;
+p->record.length=0.0;
+
 p->rforsecs=secs;
 gst_element_set_state(p->record.pipeline, GST_STATE_NULL);
 dicto_set_filename(d, p->record.sink, file);
@@ -1460,6 +1496,38 @@ dicto_delete(Dicto *d, const gchar *file)
 return dicto_delete_full(d, file, NULL);
 }
 
+/**
+ * dicto_pause:
+ *
+ * Pause playback. Emits signal #Dicto::paused_play if pipeline was paused.
+ *
+ * Returns: TRUE if a pipeline was paused or was already in paused state. FALSE otherwise.
+ */
+gboolean
+dicto_pause(Dicto *d)
+{
+DictoPrivate *p=GET_PRIVATE(d);
+GstState current, pending;
+
+if (p->play.pipeline && p->play.active) {
+	gst_element_get_state(p->play.pipeline, &current, &pending, GST_CLOCK_TIME_NONE);
+	switch (current) {
+	case GST_STATE_NULL:
+		return FALSE;
+	case GST_STATE_READY:
+	case GST_STATE_PAUSED:
+	case GST_STATE_PLAYING:
+		dicto_position_update(d, FALSE);
+		dicto_get_position(d, NULL, NULL);
+		gst_element_set_state(p->play.pipeline, GST_STATE_PAUSED);
+		p->state=DICTO_STATE_PAUSED;
+		g_signal_emit(d, signals[SIGNAL_PAUSED_PLAY], 0, NULL);
+		return TRUE;
+	}
+}
+return FALSE;
+}
+
 /**
  * dicto_stop:
  * @d: A #Dicto object
@@ -1577,25 +1645,41 @@ dicto_get_position(Dicto *d, gdouble *length, gdouble *position)
 DictoPrivate *p=GET_PRIVATE(d);
 GstFormat fmt=GST_FORMAT_TIME;
 GstElement *pipe;
+dicto_pipeline *dp;
 gint64 pos, len;
-gboolean r=FALSE;
+gboolean r;
+GstState current, pending;
 
 g_return_val_if_fail(d, FALSE);
 
-pipe=(p->record.active) ? p->record.pipeline : p->play.pipeline;
+dp=(p->record.active) ? &p->record : &p->play;
+pipe=dp->pipeline;
+g_return_val_if_fail(pipe, FALSE);
 
-if (!pipe)
+r=gst_element_query_duration(pipe, &fmt, &len);
+dp->length=GST_TIME_TO_SECS(len);
+
+if (length)
+	*length=dp->length;
+
+if (gst_element_get_state(pipe, &current, &pending, GST_CLOCK_TIME_NONE)==GST_STATE_CHANGE_FAILURE)
 	return FALSE;
 
-if (length!=NULL && gst_element_query_duration(pipe, &fmt, &len)) {
-	*length=GST_TIME_TO_SECS(len);
-	r=TRUE;
-}
+switch (current) {
+	case GST_STATE_PAUSED:
 
-if (position!=NULL && gst_element_query_position(pipe, &fmt, &pos)) {
-	*position=GST_TIME_TO_SECS(pos);
-	r=TRUE;
+	break;
+	case GST_STATE_READY:
+	case GST_STATE_PLAYING:
+		r=gst_element_query_position(pipe, &fmt, &pos);
+		dp->position=GST_TIME_TO_SECS(pos);
+	break;
+	case GST_STATE_NULL:
+	default:
+	break;
 }
+if (position)
+	*position=dp->position;
 
 return r;
 }
